@@ -1,9 +1,9 @@
-module GHC.Util.DynFlags (baseDynFlags) where
+module GHC.Util.DynFlags (initGlobalDynFlags, baseDynFlags) where
 
 import DynFlags
 import GHC.LanguageExtensions.Type
 import Data.List.Extra
-import Language.Haskell.GhclibParserEx.Config
+import Language.Haskell.GhclibParserEx.GHC.Settings.Config
 
 baseDynFlags :: DynFlags
 baseDynFlags =
@@ -17,3 +17,7 @@ baseDynFlags =
   --    hlint.yaml:860
   let enable = [TemplateHaskellQuotes]
   in foldl' xopt_set (defaultDynFlags fakeSettings fakeLlvmConfig) enable
+
+
+initGlobalDynFlags :: IO ()
+initGlobalDynFlags = setUnsafeGlobalDynFlags baseDynFlags
