@@ -1,7 +1,7 @@
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 
 module GHC.Util.SrcLoc (
-    stripLocs'
+    stripLocs
   , SrcSpanD(..)
   ) where
 
@@ -10,12 +10,12 @@ import Outputable
 
 import Data.Default
 import Data.Data
-import Data.Generics.Uniplate.Data
+import Data.Generics.Uniplate.DataOnly
 
 -- 'stripLocs x' is 'x' with all contained source locs replaced by
 -- 'noSrcSpan'.
-stripLocs' :: (Data from, HasSrcSpan from) => from -> from
-stripLocs' = transformBi (const noSrcSpan)
+stripLocs :: (Data from, HasSrcSpan from) => from -> from
+stripLocs = transformBi (const noSrcSpan)
 
 -- 'Duplicates.hs' requires 'SrcSpan' be in 'Default'.
 newtype SrcSpanD = SrcSpanD SrcSpan
