@@ -35,6 +35,13 @@ no = ($1)
 yes = (($1)) -- @Warning ($1)
 no = (+5)
 yes = ((+5)) -- @Warning (+5)
+issue909 = case 0 of { _ | n <- (0 :: Int) -> n }
+issue909 = foo (\((x :: z) -> y) -> 9 + x * 7)
+issue909 = foo (\((x : z) -> y) -> 9 + x * 7) -- \(x : z -> y) -> 9 + x * 7
+issue909 = let ((x:: y) -> z) = q in q
+issue909 = do {((x :: y) -> z) <- e; return 1}
+issue970 = (f x +) (g x) -- f x + (g x) @NoRefactor
+issue969 = (Just \x -> x || x) *> Just True
 
 -- type bracket reduction
 foo :: (Int -> Int) -> Int
@@ -56,6 +63,7 @@ yes = white $ keysymbol -- white keysymbol
 yes = operator foo $ operator -- operator foo operator
 no = operator foo $ operator bar
 yes = return $ Record{a=b}
+no = f $ [1,2..5] -- f [1,2..5] @NoRefactor: apply-refact bug; see apply-refact #51
 
 -- $/bracket rotation tests
 yes = (b $ c d) ++ e -- b (c d) ++ e
@@ -83,34 +91,37 @@ bad x = x . (x +? x . x)
 special = foo $ f{x=1}
 special = foo $ Rec{x=1}
 special = foo (f{x=1})
+loadCradleOnlyonce = skipManyTill anyMessage (message @PublishDiagnosticsNotification)
 </TEST>
 -}
 
 
 module Hint.Bracket(bracketHint) where
 
-import Hint.Type(DeclHint',Idea(..),rawIdea',warn',suggest',Severity(..),toSS')
+import Hint.Type(DeclHint,Idea(..),rawIdea,warn,suggest,Severity(..),toRefactSrcSpan,toSS)
 import Data.Data
-import Data.Generics.Uniplate.Operations
+import Data.List.Extra
+import Data.Generics.Uniplate.DataOnly
 import Refact.Types
 
-import HsSyn
+import GHC.Hs
 import Outputable
 import SrcLoc
 import GHC.Util
 import Language.Haskell.GhclibParserEx.GHC.Hs.Expr
+import Language.Haskell.GhclibParserEx.GHC.Utils.Outputable
 
-bracketHint :: DeclHint'
+bracketHint :: DeclHint
 bracketHint _ _ x =
   concatMap (\x -> bracket prettyExpr isPartialAtom True x ++ dollar x) (childrenBi (descendBi annotations x) :: [LHsExpr GhcPs]) ++
   concatMap (bracket unsafePrettyPrint (const False) False) (childrenBi x :: [LHsType GhcPs]) ++
-  concatMap (bracket unsafePrettyPrint (const False) False) (childrenBi x :: [Pat GhcPs]) ++
+  concatMap (bracket unsafePrettyPrint (const False) False) (childrenBi x :: [LPat GhcPs]) ++
   concatMap fieldDecl (childrenBi x)
    where
      -- Brackets the roots of annotations are fine, so we strip them.
      annotations :: AnnDecl GhcPs -> AnnDecl GhcPs
      annotations= descendBi $ \x -> case (x :: LHsExpr GhcPs) of
-       LL l (HsPar _ x) -> x
+       L l (HsPar _ x) -> x
        x -> x
 
 -- If we find ourselves in the context of a section and we want to
@@ -120,8 +131,8 @@ bracketHint _ _ x =
 -- latter (in contrast to the HSE pretty printer). This patches things
 -- up.
 prettyExpr :: LHsExpr GhcPs -> String
-prettyExpr s@(LL _ SectionL{}) = unsafePrettyPrint (noLoc (HsPar noExt s) :: LHsExpr GhcPs)
-prettyExpr s@(LL _ SectionR{}) = unsafePrettyPrint (noLoc (HsPar noExt s) :: LHsExpr GhcPs)
+prettyExpr s@(L _ SectionL{}) = unsafePrettyPrint (noLoc (HsPar noExtField s) :: LHsExpr GhcPs)
+prettyExpr s@(L _ SectionR{}) = unsafePrettyPrint (noLoc (HsPar noExtField s) :: LHsExpr GhcPs)
 prettyExpr x = unsafePrettyPrint x
 
 -- Dirty, should add to Brackets type class I think
@@ -137,79 +148,78 @@ findType = tyConToRtype . dataTypeName . dataTypeOf
 
 -- 'Just _' if at least one set of parens were removed. 'Nothing' if
 -- zero parens were removed.
-remParens' :: Brackets' a => a -> Maybe a
-remParens' = fmap go . remParen'
+remParens' :: Brackets a => a -> Maybe a
+remParens' = fmap go . remParen
   where
-    go e = maybe e go (remParen' e)
+    go e = maybe e go (remParen e)
 
 isPartialAtom :: LHsExpr GhcPs -> Bool
 -- Might be '$x', which was really '$ x', but TH enabled misparsed it.
-isPartialAtom (LL _ (HsSpliceE _ (HsTypedSplice _ HasDollar _ _) )) = True
-isPartialAtom (LL _ (HsSpliceE _ (HsUntypedSplice _ HasDollar _ _) )) = True
+isPartialAtom (L _ (HsSpliceE _ (HsTypedSplice _ HasDollar _ _) )) = True
+isPartialAtom (L _ (HsSpliceE _ (HsUntypedSplice _ HasDollar _ _) )) = True
 isPartialAtom x = isRecConstr x || isRecUpdate x
 
-bracket :: forall a . (Data a, Data (SrcSpanLess a), HasSrcSpan a, Outputable a, Brackets' a) => (a -> String) -> (a -> Bool) -> Bool -> a -> [Idea]
+bracket :: forall a . (Data a, Data (SrcSpanLess a), HasSrcSpan a, Outputable a, Brackets a) => (a -> String) -> (a -> Bool) -> Bool -> a -> [Idea]
 bracket pretty isPartialAtom root = f Nothing
   where
     msg = "Redundant bracket"
-    -- 'f' is a (generic) function over types in 'Brackets'
+    -- 'f' is a (generic) function over types in 'Brackets
     -- (expressions, patterns and types). Arguments are, 'f (Maybe
     -- (index, parent, gen)) child'.
-    f :: (HasSrcSpan a, Data a, Outputable a, Brackets' a) => Maybe (Int, a , a -> a) -> a -> [Idea]
+    f :: (HasSrcSpan a, Data a, Outputable a, Brackets a) => Maybe (Int, a , a -> a) -> a -> [Idea]
     -- No context. Removing parentheses from 'x' succeeds?
     f Nothing o@(remParens' -> Just x)
       -- If at the root, or 'x' is an atom, 'x' parens are redundant.
-      | root || isAtom' x
+      | root || isAtom x
       , not $ isPartialAtom x =
-          (if isAtom' x then bracketError else bracketWarning) msg o x : g x
+          (if isAtom x then bracketError else bracketWarning) msg o x : g x
     -- In some context, removing parentheses from 'x' succeeds and 'x'
     -- is atomic?
     f Just{} o@(remParens' -> Just x)
-      | isAtom' x
+      | isAtom x
       , not $ isPartialAtom x =
           bracketError msg o x : g x
     -- In some context, removing parentheses from 'x' succeeds. Does
     -- 'x' actually need bracketing in this context?
     f (Just (i, o, gen)) v@(remParens' -> Just x)
-      | not $ needBracket' i o x, not $ isPartialAtom x =
-          rawIdea' Suggestion msg (getLoc o) (pretty o) (Just (pretty (gen x))) [] [r] : g x
+      | not $ needBracket i o x, not $ isPartialAtom x =
+          rawIdea Suggestion msg (getLoc v) (pretty o) (Just (pretty (gen x))) [] [r] : g x
       where
         typ = findType (unLoc v)
-        r = Replace typ (toSS' v) [("x", toSS' x)] "x"
+        r = Replace typ (toSS v) [("x", toSS x)] "x"
     -- Regardless of the context, there are no parentheses to remove
     -- from 'x'.
     f _ x = g x
 
-    g :: (HasSrcSpan a, Data a, Outputable a, Brackets' a) => a -> [Idea]
+    g :: (HasSrcSpan a, Data a, Outputable a, Brackets a) => a -> [Idea]
     -- Enumerate over all the immediate children of 'o' looking for
     -- redundant parentheses in each.
-    g o = concat [f (Just (i, o, gen)) x | (i, (x, gen)) <- zip [0..] $ holes o]
+    g o = concat [f (Just (i, o, gen)) x | (i, (x, gen)) <- zipFrom 0 $ holes o]
 
 bracketWarning :: (HasSrcSpan a, HasSrcSpan b, Data (SrcSpanLess b), Outputable a, Outputable b) => String -> a -> b -> Idea
 bracketWarning msg o x =
-  suggest' msg o x [Replace (findType (unLoc x)) (toSS' o) [("x", toSS' x)] "x"]
+  suggest msg o x [Replace (findType (unLoc x)) (toSS o) [("x", toSS x)] "x"]
 
 bracketError :: (HasSrcSpan a, HasSrcSpan b, Data (SrcSpanLess b), Outputable a, Outputable b ) => String -> a -> b -> Idea
 bracketError msg o x =
-  warn' msg o x [Replace (findType (unLoc x)) (toSS' o) [("x", toSS' x)] "x"]
+  warn msg o x [Replace (findType (unLoc x)) (toSS o) [("x", toSS x)] "x"]
 
 fieldDecl ::  LConDeclField GhcPs -> [Idea]
-fieldDecl o@(LL loc f@ConDeclField{cd_fld_type=v@(LL l (HsParTy _ c))}) =
-   let r = LL loc (f{cd_fld_type=c}) :: LConDeclField GhcPs in
-   [rawIdea' Suggestion "Redundant bracket" loc
+fieldDecl o@(L loc f@ConDeclField{cd_fld_type=v@(L l (HsParTy _ c))}) =
+   let r = L loc (f{cd_fld_type=c}) :: LConDeclField GhcPs in
+   [rawIdea Suggestion "Redundant bracket" l
     (showSDocUnsafe $ ppr_fld o) -- Note this custom printer!
     (Just (showSDocUnsafe $ ppr_fld r))
     []
-    [Replace Type (toSS' v) [("x", toSS' c)] "x"]]
+    [Replace Type (toSS v) [("x", toSS c)] "x"]]
    where
      -- If we call 'unsafePrettyPrint' on a field decl, we won't like
      -- the output (e.g. "[foo, bar] :: T"). Here we use a custom
      -- printer to work around (snarfed from
      -- https://hackage.haskell.org/package/ghc-lib-parser-8.8.1/docs/src/HsTypes.html#pprConDeclFields).
-     ppr_fld (LL _ ConDeclField { cd_fld_names = ns, cd_fld_type = ty, cd_fld_doc = doc })
+     ppr_fld (L _ ConDeclField { cd_fld_names = ns, cd_fld_type = ty, cd_fld_doc = doc })
        = ppr_names ns <+> dcolon <+> ppr ty <+> ppr_mbDoc doc
-     ppr_fld (LL _ (XConDeclField x)) = ppr x
-     ppr_fld _ = undefined -- '{-# COMPLETE LL #-}'
+     ppr_fld (L _ (XConDeclField x)) = ppr x
 
      ppr_names [n] = ppr n
      ppr_names ns = sep (punctuate comma (map ppr ns))
@@ -220,26 +230,32 @@ fieldDecl _ = []
 dollar :: LHsExpr GhcPs -> [Idea]
 dollar = concatMap f . universe
   where
-    f x = [ suggest' "Redundant $" x y [r]| o@(LL loc (OpApp _ a d b)) <- [x], isDol d
-            , let y = noLoc (HsApp noExt a b) :: LHsExpr GhcPs
-            , not $ needBracket' 0 y a
-            , not $ needBracket' 1 y b
+    f x = [ (suggest "Redundant $" x y [r]){ideaSpan = getLoc d} | o@(L _ (OpApp _ a d b)) <- [x], isDol d
+            , let y = noLoc (HsApp noExtField a b) :: LHsExpr GhcPs
+            , not $ needBracket 0 y a
+            , not $ needBracket 1 y b
             , not $ isPartialAtom b
-            , let r = Replace Expr (toSS' x) [("a", toSS' a), ("b", toSS' b)] "a b"]
+            , let r = Replace Expr (toSS x) [("a", toSS a), ("b", toSS b)] "a b"]
           ++
-          [ suggest' "Move brackets to avoid $" x (t y) [r]
-            |(t, e@(LL _ (HsPar _ (LL _ (OpApp _ a1 op1 a2))))) <- splitInfix x
+          [ suggest "Move brackets to avoid $" x (t y) [r]
+            |(t, e@(L _ (HsPar _ (L _ (OpApp _ a1 op1 a2))))) <- splitInfix x
             , isDol op1
-            , isVar a1 || isApp a1 || isPar a1, not $ isAtom' a2
+            , isVar a1 || isApp a1 || isPar a1, not $ isAtom a2
             , varToStr a1 /= "select" -- special case for esqueleto, see #224
-            , let y = noLoc $ HsApp noExt a1 (noLoc (HsPar noExt a2))
-            , let r = Replace Expr (toSS' e) [("a", toSS' a1), ("b", toSS' a2)] "a (b)" ]
+            , let y = noLoc $ HsApp noExtField a1 (noLoc (HsPar noExtField a2))
+            , let r = Replace Expr (toSS e) [("a", toSS a1), ("b", toSS a2)] "a (b)" ]
           ++  -- Special case of (v1 . v2) <$> v3
-          [ suggest' "Redundant bracket" x y []
-          | LL _ (OpApp _ (LL _ (HsPar _ o1@(LL _ (OpApp _ v1 (isDot -> True) v2)))) o2 v3) <- [x], varToStr o2 == "<$>"
-          , let y = noLoc (OpApp noExt o1 o2 v3) :: LHsExpr GhcPs]
+          [ (suggest "Redundant bracket" x y [r]){ideaSpan = locPar}
+          | L _ (OpApp _ (L locPar (HsPar _ o1@(L locNoPar (OpApp _ v1 (isDot -> True) v2)))) o2 v3) <- [x], varToStr o2 == "<$>"
+          , let y = noLoc (OpApp noExtField o1 o2 v3) :: LHsExpr GhcPs
+          , let r = Replace Expr (toRefactSrcSpan locPar) [("a", toRefactSrcSpan locNoPar)] "a"]
+          ++
+          [ suggest "Redundant section" x y []
+          | L _ (HsApp _ (L _ (HsPar _ (L _ (SectionL _ a b)))) c) <- [x]
+          -- , error $ show (unsafePrettyPrint a, gshow b, unsafePrettyPrint c)
+          , let y = noLoc $ OpApp noExtField a b c :: LHsExpr GhcPs]
 
 splitInfix :: LHsExpr GhcPs -> [(LHsExpr GhcPs -> LHsExpr GhcPs, LHsExpr GhcPs)]
-splitInfix (LL l (OpApp _ lhs op rhs)) =
-  [(LL l . OpApp noExt lhs op, rhs), (\lhs -> LL l (OpApp noExt lhs op rhs), lhs)]
+splitInfix (L l (OpApp _ lhs op rhs)) =
+  [(L l . OpApp noExtField lhs op, rhs), (\lhs -> L l (OpApp noExtField lhs op rhs), lhs)]
 splitInfix _ = []
