@@ -1,4 +1,4 @@
-{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE LambdaCase, NamedFieldPuns #-}
 
 {-
     Suggest removal of unnecessary extensions
@@ -33,6 +33,33 @@ data Set (cxt :: * -> *) a = Set [a]
 foo x = let !y = x in y
 {-# LANGUAGE BangPatterns #-} \
 data Foo = Foo !Int --
+{-# LANGUAGE TypeOperators #-} \
+data (<+>) a b = Foo a b
+{-# LANGUAGE TypeOperators #-} \
+data Foo a b = a :+ b --
+{-# LANGUAGE TypeOperators #-} \
+type (<+>) a b = Foo a b
+{-# LANGUAGE TypeOperators #-} \
+type Foo a b = a :+ b
+{-# LANGUAGE TypeOperators, TypeFamilies #-} \
+type family Foo a b :: Type where Foo a b = a :+ b
+{-# LANGUAGE TypeOperators, TypeFamilies #-} \
+type family Foo a b :: Type where Foo a b = (<+>) a b -- {-# LANGUAGE TypeFamilies #-}
+{-# LANGUAGE TypeOperators, TypeFamilies #-} \
+class Foo a where data (<+>) a
+{-# LANGUAGE TypeOperators, TypeFamilies #-} \
+class Foo a where foo :: a -> Int <+> Bool
+{-# LANGUAGE TypeOperators #-} \
+class (<+>) a where
+{-# LANGUAGE TypeOperators #-} \
+foo :: Int -> Double <+> Bool \
+foo x = y
+{-# LANGUAGE TypeOperators #-} \
+foo :: Int -> (<+>) Double Bool \
+foo x = y --
+{-# LANGUAGE TypeOperators #-} \
+(<+>) :: Int -> Int -> Int \
+x <+> y = x + y --
 {-# LANGUAGE RecordWildCards #-} \
 record field = Record{..}
 {-# LANGUAGE RecordWildCards #-} \
@@ -78,6 +105,13 @@ f :: x -> (x, x); f x = (x, x) --
 f x = case x of (# a, b #) -> a
 {-# LANGUAGE GeneralizedNewtypeDeriving,UnboxedTuples #-} \
 newtype T m a = T (m a) deriving (PrimMonad)
+{-# LANGUAGE InstanceSigs #-} \
+instance Eq a => Eq (T a) where \
+  (==) :: T a -> T a -> Bool \
+  (==) (T x) (T y) = x==y
+{-# LANGUAGE InstanceSigs #-} \
+instance Eq a => Eq (T a) where \
+  (==) (T x) (T y) = x==y --
 {-# LANGUAGE DefaultSignatures #-} \
 class Val a where; val :: a --
 {-# LANGUAGE DefaultSignatures #-} \
@@ -104,6 +138,16 @@ main = id --
 main = "test"
 {-# LANGUAGE OverloadedStrings #-} \
 main = id --
+{-# LANGUAGE OverloadedLists #-} \
+main = [1]
+{-# LANGUAGE OverloadedLists #-} \
+main [1] = True
+{-# LANGUAGE OverloadedLists #-} \
+main = id --
+{-# LANGUAGE OverloadedLabels #-} \
+main = #foo
+{-# LANGUAGE OverloadedLabels #-} \
+main = id --
 {-# LANGUAGE DeriveAnyClass #-} \
 main = id --
 {-# LANGUAGE DeriveAnyClass #-} \
@@ -135,53 +179,100 @@ main = case () of x -> x --
 data Set (cxt :: * -> *) a = Set [a] -- @Note Extension KindSignatures is implied by PolyKinds
 {-# LANGUAGE QuasiQuotes, OverloadedStrings #-} \
 main = putStrLn [f|{T.intercalate "blah" []}|]
+{-# LANGUAGE NamedFieldPuns #-} \
+foo = x{bar}
+{-# LANGUAGE PatternSynonyms #-} \
+module Foo (pattern Bar) where x = 42
+{-# LANGUAGE PatternSynonyms #-} \
+import Foo (pattern Bar); x = 42
+{-# LANGUAGE PatternSynonyms #-} \
+pattern Foo s <- Bar s _ where Foo s = Bar s s
+{-# LANGUAGE PatternSynonyms #-} \
+x = 42 --
+{-# LANGUAGE MultiWayIf #-} \
+x = if | b1 -> v1 | b2 -> v2 | otherwise -> v3
+{-# LANGUAGE MultiWayIf #-} \
+x = if b1 then v1 else if b2 then v2 else v3 --
+static = 42
+{-# LANGUAGE NamedFieldPuns #-} \
+foo Foo{x} = x
+{-# LANGUAGE NamedFieldPuns #-} \
+foo = Foo{x}
+{-# LANGUAGE NamedFieldPuns #-} \
+foo = bar{x}
+{-# LANGUAGE NamedFieldPuns #-} --
+{-# LANGUAGE NumericUnderscores #-} \
+lessThanPi = (< 3.141_592_653_589_793)
+{-# LANGUAGE NumericUnderscores #-} \
+oneMillion = 0xf4__240
+{-# LANGUAGE NumericUnderscores #-} \
+avogadro = 6.022140857e+23 --
+{-# LANGUAGE StaticPointers #-} \
+static = 42 --
+{-# LANGUAGE Trustworthy #-}
+{-# LANGUAGE Trustworthy, NamedFieldPuns #-} -- {-# LANGUAGE Trustworthy #-}
+{-# LANGUAGE Haskell2010 #-}
+{-# LANGUAGE NoStarIsType, ExplicitNamespaces #-} \
+import GHC.TypeLits(KnownNat, type (+), type (*))
+{-# LANGUAGE LambdaCase, MultiWayIf, NoRebindableSyntax #-} \
+foo = \case True -> 3 -- {-# LANGUAGE LambdaCase, NoRebindableSyntax #-}
 </TEST>
 -}
 
 
 module Hint.Extensions(extensionsHint) where
 
-import Hint.Type(ModuHint, rawIdea',Severity(Warning),Note(..),toSS',ghcAnnotations,ghcModule,extensionImpliedBy,extensionImplies)
-import Language.Haskell.Exts.Extension
+import Hint.Type(ModuHint, rawIdea,Severity(Warning),Note(..),toSS,ghcAnnotations,ghcModule)
+import Extension
 
-import Data.Generics.Uniplate.Operations
+import Data.Generics.Uniplate.DataOnly
 import Control.Monad.Extra
+import Data.Maybe
 import Data.List.Extra
-import Data.Ratio
 import Data.Data
 import Refact.Types
 import qualified Data.Set as Set
 import qualified Data.Map as Map
 
 import SrcLoc
-import HsSyn
+import GHC.Hs
 import BasicTypes
 import Class
 import RdrName
-import OccName
 import ForeignCall
+
 import GHC.Util
+import GHC.LanguageExtensions.Type
+
+import Language.Haskell.GhclibParserEx.GHC.Hs.Pat
 import Language.Haskell.GhclibParserEx.GHC.Hs.Expr
+import Language.Haskell.GhclibParserEx.GHC.Hs.Types
+import Language.Haskell.GhclibParserEx.GHC.Hs.Decls
+import Language.Haskell.GhclibParserEx.GHC.Hs.Binds
+import Language.Haskell.GhclibParserEx.GHC.Hs.ImpExp
+import Language.Haskell.GhclibParserEx.GHC.Driver.Session
+import Language.Haskell.GhclibParserEx.GHC.Utils.Outputable
+import Language.Haskell.GhclibParserEx.GHC.Types.Name.Reader
 
 extensionsHint :: ModuHint
 extensionsHint _ x =
-    [ rawIdea' Hint.Type.Warning "Unused LANGUAGE pragma"
+    [ rawIdea Hint.Type.Warning "Unused LANGUAGE pragma"
         sl
-        (comment (mkLangExts sl exts))
+        (comment (mkLanguagePragmas sl exts))
         (Just newPragma)
-        ( [RequiresExtension $ prettyExtension gone | x <- before \\ after, gone <- Map.findWithDefault [] x disappear] ++
-            [ Note $ "Extension " ++ prettyExtension x ++ " is " ++ reason x
-            | x <- explainedRemovals])
-        [ModifyComment (toSS' (mkLangExts sl exts)) newPragma]
-    | (LL sl _,  exts) <- langExts $ pragmas (ghcAnnotations x)
-    , let before = map parseExtension exts
-    , let after = filter (`Set.member` keep) before
+        ( [RequiresExtension (show gone) | (_, Just x) <- before \\ after, gone <- Map.findWithDefault [] x disappear] ++
+            [ Note $ "Extension " ++ show x ++ " is " ++ reason x
+            | (_, Just x) <- explainedRemovals])
+        [ModifyComment (toSS (mkLanguagePragmas sl exts)) newPragma]
+    | (L sl _,  exts) <- languagePragmas $ pragmas (ghcAnnotations x)
+    , let before = [(x, readExtension x) | x <- exts]
+    , let after = filter (maybe True (`Set.member` keep) . snd) before
     , before /= after
     , let explainedRemovals
-            | null after && not (any (`Map.member` implied) before) = []
+            | null after && not (any (`Map.member` implied) $ mapMaybe snd before) = []
             | otherwise = before \\ after
     , let newPragma =
-            if null after then "" else comment (mkLangExts sl $ map prettyExtension after)
+            if null after then "" else comment (mkLanguagePragmas sl $ map fst after)
     ]
   where
     usedTH :: Bool
@@ -191,9 +282,9 @@ extensionsHint _ x =
 
     -- All the extensions defined to be used.
     extensions :: Set.Set Extension
-    extensions = Set.fromList [ parseExtension e
-                              | let exts = concatMap snd $ langExts (pragmas (ghcAnnotations x))
-                              , e <- exts ]
+    extensions = Set.fromList $ mapMaybe readExtension $
+        concatMap snd $ languagePragmas (pragmas (ghcAnnotations x))
+
     -- Those extensions we detect to be useful.
     useful :: Set.Set Extension
     useful = if usedTH then extensions else Set.filter (`usedExt` ghcModule x) extensions
@@ -203,25 +294,27 @@ extensionsHint _ x =
     implied = Map.fromList
         [ (e, a)
         | e <- Set.toList useful
-        , a:_ <- [filter (`Set.member` useful) $ extensionImpliedBy e]]
+        , a:_ <- [filter (`Set.member` useful) $ extensionImpliedEnabledBy e]
+        ]
     -- Those we should keep.
     keep :: Set.Set Extension
     keep =  useful `Set.difference` Map.keysSet implied
     -- The meaning of (a,b) is a used to imply b, but has gone, so
     -- suggest enabling b.
+    disappear :: Map.Map Extension [Extension]
     disappear =
         Map.fromListWith (++) $
         nubOrdOn snd -- Only keep one instance for each of a.
         [ (e, [a])
         | e <- Set.toList $ extensions `Set.difference` keep
-        , a <- extensionImplies e
+        , a <- fst $ extensionImplies e
         , a `Set.notMember` useful
         , usedTH || usedExt a (ghcModule x)
         ]
     reason :: Extension -> String
     reason x =
       case Map.lookup x implied of
-        Just a -> "implied by " ++ prettyExtension a
+        Just a -> "implied by " ++ show a
         Nothing -> "not used"
 
 deriveHaskell = ["Eq","Ord","Enum","Ix","Bounded","Read","Show"]
@@ -238,13 +331,14 @@ deriveStock :: [String]
 deriveStock = deriveHaskell ++ deriveGenerics ++ deriveCategory
 
 usedExt :: Extension -> Located (HsModule GhcPs) -> Bool
-usedExt (EnableExtension x) = used x
-usedExt (UnknownExtension "NumDecimals") = hasS isWholeFrac
-usedExt (UnknownExtension "DeriveLift") = hasDerive ["Lift"]
-usedExt (UnknownExtension "DeriveAnyClass") = not . null . derivesAnyclass . derives
-usedExt _ = const True
+usedExt NumDecimals = hasS isWholeFrac
+  -- Only whole number fractions are permitted by NumDecimals
+  -- extension.  Anything not-whole raises an error.
+usedExt DeriveLift = hasDerive ["Lift"]
+usedExt DeriveAnyClass = not . null . derivesAnyclass . derives
+usedExt x = used x
 
-used :: KnownExtension -> Located (HsModule GhcPs) -> Bool
+used :: Extension -> Located (HsModule GhcPs) -> Bool
 used RecursiveDo = hasS isMDo ||^ hasS isRecStmt
 used ParallelListComp = hasS isParComp
 used FunctionalDependencies = hasT (un :: FunDep (Located RdrName))
@@ -258,78 +352,108 @@ used EmptyDataDecls = hasS f
 used EmptyCase = hasS f
   where
     f :: HsExpr GhcPs -> Bool
-    f (HsCase _ _ (MG _ (LL _ []) _)) = True
-    f (HsLamCase _ (MG _ (LL _ []) _)) = True
+    f (HsCase _ _ (MG _ (L _ []) _)) = True
+    f (HsLamCase _ (MG _ (L _ []) _)) = True
     f _ = False
 used KindSignatures = hasT (un :: HsKind GhcPs)
-used BangPatterns = hasS isPBangPat' ||^ hasS isStrictMatch
-  where
-    isStrictMatch :: HsMatchContext RdrName -> Bool
-    isStrictMatch FunRhs{mc_strictness=SrcStrict} = True
-    isStrictMatch _ = False
+used BangPatterns = hasS isPBangPat ||^ hasS isStrictMatch
 used TemplateHaskell = hasT2' (un :: (HsBracket GhcPs, HsSplice GhcPs)) ||^ hasS f ||^ hasS isSpliceDecl
-    where
-      f :: HsBracket GhcPs -> Bool
-      f VarBr{} = True
-      f TypBr{} = True
-      f _ = False
+  where
+    f :: HsBracket GhcPs -> Bool
+    f VarBr{} = True
+    f TypBr{} = True
+    f _ = False
 used ForeignFunctionInterface = hasT (un :: CCallConv)
 used PatternGuards = hasS f
   where
     f :: GRHS GhcPs (LHsExpr GhcPs) -> Bool
     f (GRHS _ xs _) = g xs
-    f _ = False -- new ctor
+    f _ = False -- Extension constructor
     g :: [GuardLStmt GhcPs] -> Bool
     g [] = False
-    g [LL _ BodyStmt{}] = False
+    g [L _ BodyStmt{}] = False
     g _ = True
-used StandaloneDeriving = hasS isDerivD'
-used PatternSignatures = hasS isPatTypeSig'
-used RecordWildCards = hasS hasFieldsDotDot ||^ hasS hasPFieldsDotDot'
-used RecordPuns = hasS isPFieldPun' ||^ hasS isFieldPun
-used NamedFieldPuns = hasS isPFieldPun' ||^ hasS isFieldPun
-used UnboxedTuples = has isUnboxedTuple' ||^ has (== Unboxed) ||^ hasS isDeriving
-    where
-        -- detect if there are deriving declarations or data ... deriving stuff
-        -- by looking for the deriving strategy both contain (even if its Nothing)
-        -- see https://github.com/ndmitchell/hlint/issues/833 for why we care
-        isDeriving :: Maybe (LDerivStrategy GhcPs) -> Bool
-        isDeriving _ = True
+used StandaloneDeriving = hasS isDerivD
+used TypeOperators = hasS tyOpInSig ||^ hasS tyOpInDecl
+  where
+    tyOpInSig :: HsType GhcPs -> Bool
+    tyOpInSig = \case
+      HsOpTy{} -> True; _ -> False
+
+    tyOpInDecl :: HsDecl GhcPs -> Bool
+    tyOpInDecl = \case
+      (TyClD _ (FamDecl _ FamilyDecl{fdLName})) -> isOp fdLName
+      (TyClD _ SynDecl{tcdLName}) -> isOp tcdLName
+      (TyClD _ DataDecl{tcdLName}) -> isOp tcdLName
+      (TyClD _ ClassDecl{tcdLName, tcdATs}) -> any isOp (tcdLName : [fdLName famDecl | L _ famDecl <- tcdATs])
+      _ -> False
+
+    isOp (L _ name) = isSymbolRdrName name
+
+used RecordWildCards = hasS hasFieldsDotDot ||^ hasS hasPFieldsDotDot
+used RecordPuns = hasS isPFieldPun ||^ hasS isFieldPun ||^ hasS isFieldPunUpdate
+used UnboxedTuples = hasS isUnboxedTuple ||^ hasS (== Unboxed) ||^ hasS isDeriving
+  where
+      -- detect if there are deriving declarations or data ... deriving stuff
+      -- by looking for the deriving strategy both contain (even if its Nothing)
+      -- see https://github.com/ndmitchell/hlint/issues/833 for why we care
+      isDeriving :: Maybe (LDerivStrategy GhcPs) -> Bool
+      isDeriving _ = True
 used PackageImports = hasS f
-    where
-        f :: ImportDecl GhcPs -> Bool
-        f ImportDecl{ideclPkgQual=Just _} = True
-        f _ = False
-used QuasiQuotes = hasS isQuasiQuote ||^ hasS isTyQuasiQuote'
-used ViewPatterns = hasS isPViewPat'
-used DefaultSignatures = hasS isClsDefSig'
+  where
+      f :: ImportDecl GhcPs -> Bool
+      f ImportDecl{ideclPkgQual=Just _} = True
+      f _ = False
+used QuasiQuotes = hasS isQuasiQuote ||^ hasS isTyQuasiQuote
+used ViewPatterns = hasS isPViewPat
+used InstanceSigs = hasS f
+  where
+    f :: HsDecl GhcPs -> Bool
+    f (InstD _ decl) = hasT (un :: Sig GhcPs) decl
+    f _ = False
+used DefaultSignatures = hasS isClsDefSig
 used DeriveDataTypeable = hasDerive ["Data","Typeable"]
 used DeriveFunctor = hasDerive ["Functor"]
 used DeriveFoldable = hasDerive ["Foldable"]
 used DeriveTraversable = hasDerive ["Traversable","Foldable","Functor"]
 used DeriveGeneric = hasDerive ["Generic","Generic1"]
 used GeneralizedNewtypeDeriving = not . null . derivesNewtype' . derives
+used MultiWayIf = hasS isMultiIf
+used NumericUnderscores = hasS f
+  where
+    f :: OverLitVal -> Bool
+    f (HsIntegral (IL (SourceText t) _ _)) = '_' `elem` t
+    f (HsFractional (FL (SourceText t) _ _)) = '_' `elem` t
+    f _ = False
+
 used LambdaCase = hasS isLCase
 used TupleSections = hasS isTupleSection
 used OverloadedStrings = hasS isString
-used Arrows = hasS f
+used OverloadedLists = hasS isListExpr ||^ hasS isListPat
   where
-    f :: HsExpr GhcPs -> Bool
-    f HsProc{} = True
-    f HsArrApp{} = True
-    f _ = False
-used TransformListComp = hasS f
-    where
-      f :: StmtLR GhcPs GhcPs (LHsExpr GhcPs) -> Bool
-      f TransStmt{} = True
-      f _ = False
+    isListExpr :: HsExpr GhcPs -> Bool
+    isListExpr ExplicitList{} = True
+    isListExpr ArithSeq{} = True
+    isListExpr _ = False
+
+    isListPat :: Pat GhcPs -> Bool
+    isListPat ListPat{} = True
+    isListPat _ = False
+
+used OverloadedLabels = hasS isLabel
+  where
+    isLabel :: HsExpr GhcPs -> Bool
+    isLabel HsOverLabel{} = True
+    isLabel _ = False
+
+used Arrows = hasS isProc
+used TransformListComp = hasS isTransStmt
 used MagicHash = hasS f ||^ hasS isPrimLiteral
-    where
-      f :: RdrName -> Bool
-      f s = "#" `isSuffixOf` (occNameString . rdrNameOcc) s
--- For forwards compatibility, if things ever get added to the
--- extension enumeration.
-used x = usedExt $ UnknownExtension $ show x
+  where
+    f :: RdrName -> Bool
+    f s = "#" `isSuffixOf` occNameStr s
+used PatternSynonyms = hasS isPatSynBind ||^ hasS isPatSynIE
+used _= const True
 
 hasDerive :: [String] -> Located (HsModule GhcPs) -> Bool
 hasDerive want = any (`elem` want) . derivesStock' . derives
@@ -357,38 +481,35 @@ addDerives _ (Just s) xs = case s of
 addDerives nt _ xs = mempty
     {derivesStock' = stock
     ,derivesAnyclass = other
-    ,derivesNewtype' = if maybe True isNewType' nt then filter (`notElem` noDeriveNewtype) xs else []}
+    ,derivesNewtype' = if maybe True isNewType nt then filter (`notElem` noDeriveNewtype) xs else []}
     where (stock, other) = partition (`elem` deriveStock) xs
 
 derives :: Located (HsModule GhcPs) -> Derives
-derives (LL _ m) =  mconcat $ map decl (childrenBi m) ++ map idecl (childrenBi m)
+derives (L _ m) =  mconcat $ map decl (childrenBi m) ++ map idecl (childrenBi m)
   where
     idecl :: Located (DataFamInstDecl GhcPs) -> Derives
-    idecl (LL _ (DataFamInstDecl (HsIB _ FamEqn {feqn_rhs=HsDataDefn {dd_ND=dn, dd_derivs=(LL _ ds)}}))) = g dn ds
+    idecl (L _ (DataFamInstDecl (HsIB _ FamEqn {feqn_rhs=HsDataDefn {dd_ND=dn, dd_derivs=(L _ ds)}}))) = g dn ds
     idecl _ = mempty
 
     decl :: LHsDecl GhcPs -> Derives
-    decl (LL _ (TyClD _ (DataDecl _ _ _ _ HsDataDefn {dd_ND=dn, dd_derivs=(LL _ ds)}))) = g dn ds -- Data declaration.
-    decl (LL _ (DerivD _ (DerivDecl _ (HsWC _ sig) strategy _))) = addDerives Nothing (fmap unLoc strategy) [derivedToStr sig] -- A deriving declaration.
+    decl (L _ (TyClD _ (DataDecl _ _ _ _ HsDataDefn {dd_ND=dn, dd_derivs=(L _ ds)}))) = g dn ds -- Data declaration.
+    decl (L _ (DerivD _ (DerivDecl _ (HsWC _ sig) strategy _))) = addDerives Nothing (fmap unLoc strategy) [derivedToStr sig] -- A deriving declaration.
     decl _ = mempty
 
     g :: NewOrData -> [LHsDerivingClause GhcPs] -> Derives
-    g dn ds = mconcat [addDerives (Just dn) (fmap unLoc strategy) $ map derivedToStr tys | LL _ (HsDerivingClause _ strategy (LL _ tys)) <- ds]
+    g dn ds = mconcat [addDerives (Just dn) (fmap unLoc strategy) $ map derivedToStr tys | L _ (HsDerivingClause _ strategy (L _ tys)) <- ds]
 
     derivedToStr :: LHsSigType GhcPs -> String
     derivedToStr (HsIB _ t) = ih t
       where
         ih :: LHsType GhcPs -> String
-        ih (LL _ (HsQualTy _ _ a)) = ih a
-        ih (LL _ (HsParTy _ a)) = ih a
-        ih (LL _ (HsAppTy _ a _)) = ih a
-        ih (LL _ (HsTyVar _ _ a)) = unsafePrettyPrint $ unqual' a
-        ih (LL _ a) = unsafePrettyPrint a -- I don't anticipate this case is called.
-        ih _ = "" -- {-# COMPLETE LL #-}
+        ih (L _ (HsQualTy _ _ a)) = ih a
+        ih (L _ (HsParTy _ a)) = ih a
+        ih (L _ (HsAppTy _ a _)) = ih a
+        ih (L _ (HsTyVar _ _ a)) = unsafePrettyPrint $ unqual a
+        ih (L _ a) = unsafePrettyPrint a -- I don't anticipate this case is called.
     derivedToStr _ = "" -- new ctor
 
-derives _ = mempty -- {-# COMPLETE LL #-}
-
 un = undefined
 
 hasT t x = not $ null (universeBi x `asTypeOf` [t])
@@ -396,12 +517,3 @@ hasT2' ~(t1,t2) = hasT t1 ||^ hasT t2
 
 hasS :: (Data x, Data a) => (a -> Bool) -> x -> Bool
 hasS test = any test . universeBi
-
-has f = any f . universeBi
-
--- Only whole number fractions are permitted by NumDecimals extension.
--- Anything not-whole raises an error.
-isWholeFrac :: HsExpr GhcPs -> Bool
-isWholeFrac (HsLit _ (HsRat _ (FL _ _ v) _)) = denominator v == 1
-isWholeFrac (HsOverLit _ (OverLit _ (HsFractional (FL _ _ v)) _)) = denominator v == 1
-isWholeFrac _ = False
