@@ -1,4 +1,4 @@
-{-# LANGUAGE PatternGuards, ScopedTypeVariables, RecordWildCards #-}
+{-# LANGUAGE LambdaCase, PatternGuards, RecordWildCards #-}
 {-
     Reduce the number of import declarations.
     Two import declarations can be combined if:
@@ -28,34 +28,32 @@ import A; import B; import A -- import A
 import qualified A; import A
 import B; import A; import A -- import A
 import A hiding(Foo); import A hiding(Bar)
-import List -- import Data.List
-import qualified List -- import qualified Data.List as List
-import Char(foo) -- import Data.Char(foo)
-import IO(foo)
-import IO as X -- import System.IO as X; import System.IO.Error as X; import Control.Exception  as X (bracket,bracket_)
+import A (foo) \
+import A (bar) \
+import A (baz) -- import A ( foo, bar, baz )
 </TEST>
 -}
 
 
 module Hint.Import(importHint) where
 
-import Hint.Type(ModuHint,ModuleEx(..),Idea(..),Severity(..),suggest',toSS',rawIdea',rawIdeaN')
+import Hint.Type(ModuHint,ModuleEx(..),Idea(..),Severity(..),suggest,toSS,rawIdea)
 import Refact.Types hiding (ModuleName)
 import qualified Refact.Types as R
 import Data.Tuple.Extra
 import Data.List.Extra
-import Data.Generics.Uniplate.Operations
+import Data.Generics.Uniplate.DataOnly
 import Data.Maybe
 import Control.Applicative
 import Prelude
 
 import FastString
 import BasicTypes
-import RdrName
-import Module
-import HsSyn
+import GHC.Hs
 import SrcLoc
-import GHC.Util
+
+import Language.Haskell.GhclibParserEx.GHC.Utils.Outputable
+
 
 importHint :: ModuHint
 importHint _ ModuleEx {ghcModule=L _ HsModule{hsmodImports=ms}} =
@@ -67,15 +65,12 @@ importHint _ ModuleEx {ghcModule=L _ HsModule{hsmodImports=ms}} =
               , let n = unLoc $ ideclName i'
               , let pkg  = unpackFS . sl_fs <$> ideclPkgQual i']) ++
   -- Ideas for removing redundant 'as' clauses.
-  concatMap stripRedundantAlias ms ++
-  -- Ideas for replacing deprecated imports by their preferred
-  -- equivalents.
-  concatMap preferHierarchicalImports ms
+  concatMap stripRedundantAlias ms
 
 reduceImports :: [LImportDecl GhcPs] -> [Idea]
 reduceImports [] = []
 reduceImports ms@(m:_) =
-  [rawIdea' Hint.Type.Warning "Use fewer imports" (getLoc m) (f ms) (Just $ f x) [] rs
+  [rawIdea Hint.Type.Warning "Use fewer imports" (getLoc m) (f ms) (Just $ f x) [] rs
   | Just (x, rs) <- [simplify ms]]
   where f = unlines . map unsafePrettyPrint
 
@@ -84,7 +79,9 @@ simplify :: [LImportDecl GhcPs]
 simplify [] = Nothing
 simplify (x : xs) = case simplifyHead x xs of
     Nothing -> first (x:) <$> simplify xs
-    Just (xs, rs) -> Just $ maybe (xs, rs) (second (++ rs)) $ simplify xs
+    Just (xs, rs) ->
+      let deletions = filter (\case Delete{} -> True; _ -> False) rs
+       in Just $ maybe (xs, rs) (second (++ deletions)) $ simplify xs
 
 simplifyHead :: LImportDecl GhcPs
              -> [LImportDecl GhcPs]
@@ -97,27 +94,27 @@ simplifyHead x [] = Nothing
 combine :: LImportDecl GhcPs
         -> LImportDecl GhcPs
         -> Maybe (LImportDecl GhcPs, [Refactoring R.SrcSpan])
-combine x@(LL _ x') y@(LL _ y')
+combine x@(L loc x') y@(L _ y')
   -- Both (un/)qualified, common 'as', same names : Delete the second.
-  | qual, as, specs = Just (x, [Delete Import (toSS' y)])
+  | qual, as, specs = Just (x, [Delete Import (toSS y)])
     -- Both (un/)qualified, common 'as', different names : Merge the
     -- second into the first and delete it.
   | qual, as
   , Just (False, xs) <- ideclHiding x'
   , Just (False, ys) <- ideclHiding y' =
-      let newImp = noLoc x'{ideclHiding = Just (False, noLoc (unLoc xs ++ unLoc ys))}
-      in Just (newImp, [Replace Import (toSS' x) [] (unsafePrettyPrint (unLoc newImp))
-                       , Delete Import (toSS' y)])
+      let newImp = L loc x'{ideclHiding = Just (False, noLoc (unLoc xs ++ unLoc ys))}
+      in Just (newImp, [Replace Import (toSS x) [] (unsafePrettyPrint (unLoc newImp))
+                       , Delete Import (toSS y)])
   -- Both (un/qualified), common 'as', one has names the other doesn't
   -- : Delete the one with names.
   | qual, as, isNothing (ideclHiding x') || isNothing (ideclHiding y') =
        let (newImp, toDelete) = if isNothing (ideclHiding x') then (x, y) else (y, x)
-       in Just (newImp, [Delete Import (toSS' toDelete)])
+       in Just (newImp, [Delete Import (toSS toDelete)])
   -- Both unqualified, same names, one (and only one) has an 'as'
   -- clause : Delete the one without an 'as'.
-  | not (ideclQualified x'), qual, specs, length ass == 1 =
+  | ideclQualified x' == NotQualified, qual, specs, length ass == 1 =
        let (newImp, toDelete) = if isJust (ideclAs x') then (x, y) else (y, x)
-       in Just (newImp, [Delete Import (toSS' toDelete)])
+       in Just (newImp, [Delete Import (toSS toDelete)])
   -- No hints.
   | otherwise = Nothing
     where
@@ -131,59 +128,10 @@ combine x@(LL _ x') y@(LL _ y')
         ass = mapMaybe ideclAs [x', y']
         specs = transformBi (const noSrcSpan) (ideclHiding x') ==
                     transformBi (const noSrcSpan) (ideclHiding y')
-combine _ _ = Nothing -- {-# COMPLETE LL #-}
 
 stripRedundantAlias :: LImportDecl GhcPs -> [Idea]
-stripRedundantAlias x@(LL loc i@ImportDecl {..})
+stripRedundantAlias x@(L loc i@ImportDecl {..})
   -- Suggest 'import M as M' be just 'import M'.
   | Just (unLoc ideclName) == fmap unLoc ideclAs =
-      [suggest' "Redundant as" x (cL loc i{ideclAs=Nothing} :: LImportDecl GhcPs) [RemoveAsKeyword (toSS' x)]]
+      [suggest "Redundant as" x (cL loc i{ideclAs=Nothing} :: LImportDecl GhcPs) [RemoveAsKeyword (toSS x)]]
 stripRedundantAlias _ = []
-
-preferHierarchicalImports :: LImportDecl GhcPs -> [Idea]
-preferHierarchicalImports x@(LL loc i@ImportDecl{ideclName=L _ n,ideclPkgQual=Nothing})
-  -- Suggest 'import IO' be rewritten 'import System.IO, import
-  -- System.IO.Error, import Control.Exception(bracket, bracket_)'.
-  | n == mkModuleName "IO" && isNothing (ideclHiding i) =
-      [rawIdeaN' Suggestion "Use hierarchical imports" loc
-      (trimStart $ unsafePrettyPrint i) (
-          Just $ unlines $ map (trimStart . unsafePrettyPrint)
-          [ f "System.IO" Nothing, f "System.IO.Error" Nothing
-          , f "Control.Exception" $ Just (False, noLoc [mkLIE x | x <- ["bracket","bracket_"]])]) []]
-  -- Suggest that a module import like 'Monad' should be rewritten with
-  -- its hiearchical equivalent e.g. 'Control.Monad'.
-  | Just y <- lookup (moduleNameString n) newNames =
-    let newModuleName = y ++ "." ++ moduleNameString n
-        r = [Replace R.ModuleName (toSS' x) [] newModuleName] in
-    [suggest' "Use hierarchical imports"
-     x (noLoc (desugarQual i){ideclName=noLoc (mkModuleName newModuleName)} :: LImportDecl GhcPs) r]
-  where
-    -- Substitute a new module name.
-    f a b = (desugarQual i){ideclName=noLoc (mkModuleName a), ideclHiding=b}
-    -- Wrap a literal name into an 'IE' (import/export) value.
-    mkLIE :: String -> LIE GhcPs
-    mkLIE n = noLoc $ IEVar noExt (noLoc (IEName (noLoc (mkVarUnqual (fsLit n)))))
-    -- Rewrite 'import qualified X' as 'import qualified X as X'.
-    desugarQual :: ImportDecl GhcPs -> ImportDecl GhcPs
-    desugarQual i
-      | ideclQualified i && isNothing (ideclAs i) = i{ideclAs = Just (ideclName i)}
-      | otherwise = i
-
-preferHierarchicalImports _ = []
-
-newNames :: [(String, String)]
-newNames = let (*) = flip (,) in
-    ["Control" * "Monad"
-    ,"Data" * "Char"
-    ,"Data" * "List"
-    ,"Data" * "Maybe"
-    ,"Data" * "Ratio"
-    ,"System" * "Directory"
-
-    -- Special, see bug https://code.google.com/archive/p/ndmitchell/issues/393
-    -- ,"System" * "IO"
-
-    -- Do not encourage use of old-locale/old-time over haskell98
-    -- ,"System" * "Locale"
-    -- ,"System" * "Time"
-    ]
