@@ -15,25 +15,25 @@
     Don't suggest for FFI, since they match their C names
 
 <TEST>
-data Yes = Foo | Bar'Test -- data Yes = Foo | BarTest
-data Yes = Bar | Test_Bar -- data Yes = Bar | TestBar
+data Yes = Foo | Bar'Test
+data Yes = Bar | Test_Bar -- data Yes = Bar | TestBar @NoRefactor
 data No = a :::: b
 data Yes = Foo {bar_cap :: Int}
 data No = FOO | BarBAR | BarBBar
-yes_foo = yes_foo + yes_foo -- yesFoo = ...
-yes_fooPattern Nothing = 0 -- yesFooPattern Nothing = ...
+yes_foo = yes_foo + yes_foo -- yesFoo = ... @NoRefactor
+yes_fooPattern Nothing = 0 -- yesFooPattern Nothing = ... @NoRefactor
 no = 1 where yes_foo = 2
 a -== b = 1
 myTest = 1; my_test = 1
-semiring'laws = 1 -- semiringLaws = ...
-data Yes = FOO_A | Foo_B -- data Yes = FOO_A | FooB
+semiring'laws = 1
+data Yes = FOO_A | Foo_B -- data Yes = FOO_A | FooB @NoRefactor
 case_foo = 1
 test_foo = 1
-cast_foo = 1 -- castFoo = ...
+cast_foo = 1 -- castFoo = ... @NoRefactor
 replicateM_ = 1
 _foo__ = 1
 section_1_1 = 1
-runMutator# = 1
+runMutator# = 1  @NoRefactor
 foreign import ccall hexml_node_child :: IO ()
 </TEST>
 -}
@@ -41,40 +41,42 @@ foreign import ccall hexml_node_child :: IO ()
 
 module Hint.Naming(namingHint) where
 
-import Hint.Type (Idea,DeclHint',suggest',isSym,toSrcSpan',ghcModule)
-import Data.Generics.Uniplate.Operations
+import Hint.Type (Idea,DeclHint,suggest,ghcModule)
+import Data.Generics.Uniplate.DataOnly
 import Data.List.Extra (nubOrd, isPrefixOf)
 import Data.Data
 import Data.Char
 import Data.Maybe
-import Refact.Types hiding (RType(Match))
 import qualified Data.Set as Set
 
 import BasicTypes
 import FastString
-import HsDecls
-import HsExtension
-import HsSyn
+import GHC.Hs.Decls
+import GHC.Hs.Extension
+import GHC.Hs
 import OccName
 import SrcLoc
 
+import Language.Haskell.GhclibParserEx.GHC.Hs.Decls
+import Language.Haskell.GhclibParserEx.GHC.Utils.Outputable
 import GHC.Util
 
-namingHint :: DeclHint'
+namingHint :: DeclHint
 namingHint _ modu = naming $ Set.fromList $ concatMap getNames $ hsmodDecls $ unLoc (ghcModule modu)
 
 naming :: Set.Set String -> LHsDecl GhcPs -> [Idea]
 naming seen originalDecl =
-    [ suggest' "Use camelCase"
+    [ suggest "Use camelCase"
                (shorten originalDecl)
                (shorten replacedDecl)
-               [Replace Bind (toSrcSpan' originalDecl) [] (unsafePrettyPrint replacedDecl)]
+               [ -- https://github.com/mpickering/apply-refact/issues/39
+               ]
     | not $ null suggestedNames
     ]
     where
         suggestedNames =
             [ (originalName, suggestedName)
-            | not $ isForD' originalDecl
+            | not $ isForD originalDecl
             , originalName <- nubOrd $ getNames originalDecl
             , Just suggestedName <- [suggestName originalName]
             , not $ suggestedName `Set.member` seen
@@ -82,23 +84,23 @@ naming seen originalDecl =
         replacedDecl = replaceNames suggestedNames originalDecl
 
 shorten :: LHsDecl GhcPs -> LHsDecl GhcPs
-shorten (LL locDecl (ValD ttg0 bind@(FunBind _ _ matchGroup@(MG _ (LL locMatches matches) FromSource) _ _))) =
-    LL locDecl (ValD ttg0 bind {fun_matches = matchGroup {mg_alts = LL locMatches $ map shortenMatch matches}})
-shorten (LL locDecl (ValD ttg0 bind@(PatBind _ _ grhss@(GRHSs _ rhss _) _))) =
-    LL locDecl (ValD ttg0 bind {pat_rhs = grhss {grhssGRHSs = map shortenLGRHS rhss}})
+shorten (L locDecl (ValD ttg0 bind@(FunBind _ _ matchGroup@(MG _ (L locMatches matches) FromSource) _ _))) =
+    L locDecl (ValD ttg0 bind {fun_matches = matchGroup {mg_alts = L locMatches $ map shortenMatch matches}})
+shorten (L locDecl (ValD ttg0 bind@(PatBind _ _ grhss@(GRHSs _ rhss _) _))) =
+    L locDecl (ValD ttg0 bind {pat_rhs = grhss {grhssGRHSs = map shortenLGRHS rhss}})
 shorten x = x
 
 shortenMatch :: LMatch GhcPs (LHsExpr GhcPs) -> LMatch GhcPs (LHsExpr GhcPs)
-shortenMatch (LL locMatch match@(Match _ _ _ grhss@(GRHSs _ rhss _))) =
-    LL locMatch match {m_grhss = grhss {grhssGRHSs = map shortenLGRHS rhss}}
+shortenMatch (L locMatch match@(Match _ _ _ grhss@(GRHSs _ rhss _))) =
+    L locMatch match {m_grhss = grhss {grhssGRHSs = map shortenLGRHS rhss}}
 shortenMatch x = x
 
 shortenLGRHS :: LGRHS GhcPs (LHsExpr GhcPs) -> LGRHS GhcPs (LHsExpr GhcPs)
-shortenLGRHS (LL locGRHS (GRHS ttg0 guards (LL locExpr _))) =
-    LL locGRHS (GRHS ttg0 guards (cL locExpr dots))
+shortenLGRHS (L locGRHS (GRHS ttg0 guards (L locExpr _))) =
+    L locGRHS (GRHS ttg0 guards (cL locExpr dots))
     where
         dots :: HsExpr GhcPs
-        dots = HsLit NoExt (HsString (SourceText "...") (mkFastString "..."))
+        dots = HsLit noExtField (HsString (SourceText "...") (mkFastString "..."))
 shortenLGRHS x = x
 
 getNames :: LHsDecl GhcPs -> [String]
@@ -109,13 +111,17 @@ getConstructorNames (TyClD _ (DataDecl _ _ _ _ (HsDataDefn _ _ _ _ _ cons _))) =
     concatMap (map unsafePrettyPrint . getConNames . unLoc) cons
 getConstructorNames _ = []
 
+isSym :: String -> Bool
+isSym (x:_) = not $ isAlpha x || x `elem` "_'"
+isSym _ = False
+
 suggestName :: String -> Maybe String
 suggestName original
     | isSym original || good || not (any isLower original) || any isDigit original ||
         any (`isPrefixOf` original) ["prop_","case_","unit_","test_","spec_","scprop_","hprop_"] = Nothing
     | otherwise = Just $ f original
     where
-        good = all isAlphaNum $ drp '_' $ drp '#' $ drp '\'' $ reverse $ drp '_' original
+        good = all isAlphaNum $ drp '_' $ drp '#' $ filter (/= '\'') $ reverse $ drp '_' original
         drp x = dropWhile (== x)
 
         f xs = us ++ g ys
