@@ -2,8 +2,8 @@
 
 module Idea(
     Idea(..),
-    rawIdea, rawIdea', idea, idea', suggest, suggest', warn, warn',ignore, ignore',
-    rawIdeaN, rawIdeaN', suggestN, suggestN', ignoreN, ignoreN', ignoreNoSuggestion',
+    rawIdea, idea, suggest, suggestRemove, ideaRemove, warn, ignore,
+    rawIdeaN, suggestN, ignoreNoSuggestion,
     showIdeasJson, showANSI,
     Note(..), showNotes,
     Severity(..),
@@ -11,15 +11,16 @@ module Idea(
 
 import Data.Functor
 import Data.List.Extra
-import HSE.All
 import Config.Type
 import HsColour
 import Refact.Types hiding (SrcSpan)
 import qualified Refact.Types as R
 import Prelude
-import qualified SrcLoc as GHC
-import qualified Outputable
-import qualified GHC.Util as GHC
+import SrcLoc
+import Outputable
+import GHC.Util
+
+import Language.Haskell.GhclibParserEx.GHC.Utils.Outputable
 
 -- | An idea suggest by a 'Hint'.
 data Idea = Idea
@@ -45,9 +46,9 @@ showIdeaJson idea@Idea{ideaSpan=srcSpan@SrcSpan{..}, ..} = dict
     ,("severity", str $ show ideaSeverity)
     ,("hint", str ideaHint)
     ,("file", str srcSpanFilename)
-    ,("startLine", show srcSpanStartLine)
+    ,("startLine", show srcSpanStartLine')
     ,("startColumn", show srcSpanStartColumn)
-    ,("endLine", show srcSpanEndLine)
+    ,("endLine", show srcSpanEndLine')
     ,("endColumn", show srcSpanEndColumn)
     ,("from", str ideaFrom)
     ,("to", maybe "null" str ideaTo)
@@ -71,7 +72,7 @@ showANSI = showEx <$> hsColourConsole
 
 showEx :: (String -> String) -> Idea -> String
 showEx tt Idea{..} = unlines $
-    [showSrcLoc (getPointLoc ideaSpan) ++ ": " ++ (if ideaHint == "" then "" else show ideaSeverity ++ ": " ++ ideaHint)] ++
+    [showSrcSpan ideaSpan ++ ": " ++ (if ideaHint == "" then "" else show ideaSeverity ++ ": " ++ ideaHint)] ++
     f "Found" (Just ideaFrom) ++ f "Perhaps" ideaTo ++
     ["Note: " ++ n | let n = showNotes ideaNote, n /= ""]
     where
@@ -84,73 +85,41 @@ showEx tt Idea{..} = unlines $
 rawIdea :: Severity -> String -> SrcSpan -> String -> Maybe String -> [Note]-> [Refactoring R.SrcSpan] -> Idea
 rawIdea = Idea [] []
 
-rawIdea' :: Severity -> String -> GHC.SrcSpan -> String -> Maybe String -> [Note]-> [Refactoring R.SrcSpan] -> Idea
-rawIdea' a b c = Idea [] [] a b (ghcSpanToHSE c)
-
 rawIdeaN :: Severity -> String -> SrcSpan -> String -> Maybe String -> [Note] -> Idea
 rawIdeaN a b c d e f = Idea [] [] a b c d e f []
 
-rawIdeaN' :: Severity -> String -> GHC.SrcSpan -> String -> Maybe String -> [Note] -> Idea
-rawIdeaN' a b c d e f = Idea [] [] a b (ghcSpanToHSE c) d e f []
-
-idea :: (Annotated ast, Pretty a, Pretty (ast SrcSpanInfo)) =>
-        Severity -> String -> ast SrcSpanInfo -> a -> [Refactoring R.SrcSpan] -> Idea
-idea severity hint from to = rawIdea severity hint (srcInfoSpan $ ann from) (f from) (Just $ f to) []
-    where f = trimStart . prettyPrint
-
-idea' :: (GHC.HasSrcSpan a, Outputable.Outputable a, GHC.HasSrcSpan b, Outputable.Outputable b) =>
+idea :: (HasSrcSpan a, Outputable.Outputable a, HasSrcSpan b, Outputable.Outputable b) =>
          Severity -> String -> a -> b -> [Refactoring R.SrcSpan] -> Idea
-idea' severity hint from to =
-  rawIdea severity hint (ghcSpanToHSE (GHC.getLoc from)) (GHC.unsafePrettyPrint from) (Just $ GHC.unsafePrettyPrint to) []
+idea severity hint from to =
+  rawIdea severity hint (getLoc from) (unsafePrettyPrint from) (Just $ unsafePrettyPrint to) []
 
-suggest :: (Annotated ast, Pretty a, Pretty (ast SrcSpanInfo)) =>
-           String -> ast SrcSpanInfo -> a -> [Refactoring R.SrcSpan] -> Idea
+-- Construct an Idea that suggests "Perhaps you should remove it."
+ideaRemove :: Severity -> String -> SrcSpan -> String -> [Refactoring R.SrcSpan] -> Idea
+ideaRemove severity hint span from = rawIdea severity hint span from (Just "") []
+
+suggest :: (HasSrcSpan a, Outputable.Outputable a, HasSrcSpan b, Outputable.Outputable b) =>
+            String -> a -> b -> [Refactoring R.SrcSpan] -> Idea
 suggest = idea Suggestion
 
-suggest' :: (GHC.HasSrcSpan a, Outputable.Outputable a, GHC.HasSrcSpan b, Outputable.Outputable b) =>
-            String -> a -> b -> [Refactoring R.SrcSpan] -> Idea
-suggest' = idea' Suggestion
+suggestRemove :: String -> SrcSpan -> String -> [Refactoring R.SrcSpan] -> Idea
+suggestRemove = ideaRemove Suggestion
 
-warn :: (Annotated ast, Pretty a, Pretty (ast SrcSpanInfo)) =>
-        String -> ast SrcSpanInfo -> a -> [Refactoring R.SrcSpan] -> Idea
+warn :: (HasSrcSpan a, Outputable.Outputable a, HasSrcSpan b, Outputable.Outputable b) =>
+         String -> a -> b -> [Refactoring R.SrcSpan] -> Idea
 warn = idea Warning
 
-warn' :: (GHC.HasSrcSpan a, Outputable.Outputable a, GHC.HasSrcSpan b, Outputable.Outputable b) =>
-         String -> a -> b -> [Refactoring R.SrcSpan] -> Idea
-warn' = idea' Warning
-
-ignoreNoSuggestion' :: (GHC.HasSrcSpan a, Outputable.Outputable a)
+ignoreNoSuggestion :: (HasSrcSpan a, Outputable.Outputable a)
                     => String -> a -> Idea
-ignoreNoSuggestion' hint x = rawIdeaN Ignore hint (ghcSpanToHSE (GHC.getLoc x)) (GHC.unsafePrettyPrint x) Nothing []
+ignoreNoSuggestion hint x = rawIdeaN Ignore hint (getLoc x) (unsafePrettyPrint x) Nothing []
 
-ignore :: (Annotated ast, Pretty a, Pretty (ast SrcSpanInfo)) =>
-          String -> ast SrcSpanInfo -> a -> [Refactoring R.SrcSpan] -> Idea
+ignore :: (HasSrcSpan a, Outputable.Outputable a) =>
+           String -> a -> a -> [Refactoring R.SrcSpan] -> Idea
 ignore = idea Ignore
 
-ignore' :: (GHC.HasSrcSpan a, Outputable.Outputable a) =>
-           String -> a -> a -> [Refactoring R.SrcSpan] -> Idea
-ignore' = idea' Ignore
-
-ideaN :: (Annotated ast, Pretty a, Pretty (ast SrcSpanInfo)) =>
-         Severity -> String -> ast SrcSpanInfo -> a -> Idea
+ideaN :: (HasSrcSpan a, Outputable.Outputable a) =>
+          Severity -> String -> a -> a -> Idea
 ideaN severity hint from to = idea severity hint from to []
 
-ideaN' :: (GHC.HasSrcSpan a, Outputable.Outputable a) =>
-          Severity -> String -> a -> a -> Idea
-ideaN' severity hint from to = idea' severity hint from to []
-
-suggestN :: (Annotated ast, Pretty a, Pretty (ast SrcSpanInfo)) =>
-            String -> ast SrcSpanInfo -> a -> Idea
-suggestN = ideaN Suggestion
-
-suggestN' :: (GHC.HasSrcSpan a, Outputable.Outputable a) =>
+suggestN :: (HasSrcSpan a, Outputable.Outputable a) =>
              String -> a -> a -> Idea
-suggestN' = ideaN' Suggestion
-
-ignoreN :: (Annotated ast, Pretty a, Pretty (ast SrcSpanInfo)) =>
-           String -> ast SrcSpanInfo -> a -> Idea
-ignoreN = ideaN Ignore
-
-ignoreN' :: (GHC.HasSrcSpan a, Outputable.Outputable a) =>
-           String -> a -> a -> Idea
-ignoreN' = ideaN' Ignore
+suggestN = ideaN Suggestion
