@@ -3,7 +3,7 @@ The parallel function (specialised to lists) is equivalent to:
 
 import Control.Parallel.Strategies
 parallel :: [IO [a]] -> IO [[a]]
-parallel = return . withStrategy (parList $ seqList r0) . map unsafePerformIO
+parallel = pure . withStrategy (parList $ seqList r0) . map unsafePerformIO
 
 However, this version performs about 10% slower with 2 processors in GHC 6.12.1
 -}
@@ -21,11 +21,11 @@ parallel j = if j <= 1 then parallel1 else parallelN j
 
 
 parallel1 :: [IO a] -> IO [a]
-parallel1 [] = return []
+parallel1 [] = pure []
 parallel1 (x:xs) = do
     x2 <- x
     xs2 <- unsafeInterleaveIO $ parallel1 xs
-    return $ x2:xs2
+    pure $ x2:xs2
 
 
 parallelN :: Int -> [IO a] -> IO [a]
@@ -40,7 +40,7 @@ parallelN j xs = do
         f chan = do
             v <- readChan chan
             case v of
-                Nothing -> return ()
+                Nothing -> pure ()
                 Just (m,x) -> do
                     putMVar m =<< try x
                     f chan
