@@ -1,5 +1,5 @@
 {-# LANGUAGE RecordWildCards #-}
-{-# OPTIONS_GHC -fno-warn-incomplete-patterns #-}
+{-# OPTIONS_GHC -Wno-incomplete-patterns #-}
 
 module Test.All(test) where
 
@@ -8,7 +8,10 @@ import System.Console.CmdArgs
 import Control.Monad
 import Control.Monad.IO.Class
 import Data.Char
+import Data.Either.Extra
+import Data.Foldable
 import Data.List
+import Data.Maybe
 import System.Directory
 import System.FilePath
 import Data.Functor
@@ -17,27 +20,30 @@ import Prelude
 import Config.Type
 import Config.Read
 import CmdLine
-import HSE.All
+import Refact
 import Hint.All
-import Test.Util
-import Test.InputOutput
 import Test.Annotations
+import Test.InputOutput
+import Test.Summary
 import Test.Translate
+import Test.Util
 import System.IO.Extra
+import Language.Haskell.GhclibParserEx.GHC.Utils.Outputable
 
 
 test :: Cmd -> ([String] -> IO ()) -> FilePath -> [FilePath] -> IO Int
 test CmdTest{..} main dataDir files = do
-    (failures, ideas) <- withBuffering stdout NoBuffering $ withTests $ do
+    rpath <- refactorPath (if cmdWithRefactor == "" then Nothing else Just cmdWithRefactor)
+
+    (failures, (ideas, builtins)) <- withBuffering stdout NoBuffering $ withTests $ do
         hasSrc <- liftIO $ doesFileExist "hlint.cabal"
-        useSrc <- return $ hasSrc && null files
-        testFiles <- if files /= [] then return files else do
+        let useSrc = hasSrc && null files
+        testFiles <- if files /= [] then pure files else do
             xs <- liftIO $ getDirectoryContents dataDir
-            return [dataDir </> x | x <- xs, takeExtension x `elem` [".hs",".yml",".yaml"]
-                                , not $ "HLint_" `isPrefixOf` takeBaseName x]
+            pure [dataDir </> x | x <- xs, takeExtension x `elem` [".yml",".yaml"]]
         testFiles <- liftIO $ forM testFiles $ \file -> do
-            hints <- readFilesConfig [(file, Nothing)]
-            return (file, hints ++ (if takeBaseName file /= "Test" then [] else map (Builtin . fst) builtinHints))
+            hints <- readFilesConfig [(file, Nothing),("CommandLine.yaml", Just "- group: {name: testing, enabled: true}")]
+            pure (file, hints ++ (if takeBaseName file /= "Test" then [] else map (Builtin . fst) builtinHints))
         let wrap msg act = do liftIO $ putStr (msg ++ " "); act; liftIO $ putStrLn ""
 
         liftIO $ putStrLn "Testing"
@@ -46,11 +52,13 @@ test CmdTest{..} main dataDir files = do
             config <- liftIO $ readFilesConfig [(".hlint.yaml",Nothing)]
             forM_ builtinHints $ \(name,_) -> do
                 progress
-                testAnnotations (Builtin name : if name == "Restrict" then config else []) $ "src/Hint" </> name <.> "hs"
+                testAnnotations (Builtin name : if name == "Restrict" then config else [])
+                                ("src/Hint" </> name <.> "hs")
+                                (eitherToMaybe rpath)
         when useSrc $ wrap "Input/outputs" $ testInputOutput main
 
         wrap "Hint names" $ mapM_ (\x -> do progress; testNames $ snd x) testFiles
-        wrap "Hint annotations" $ forM_ testFiles $ \(file,h) -> do progress; testAnnotations h file
+        wrap "Hint annotations" $ forM_ testFiles $ \(file,h) -> do progress; testAnnotations h file (eitherToMaybe rpath)
         let hs = [h | (file, h) <- testFiles, takeFileName file /= "Test.hs"]
         when cmdTypeCheck $ wrap "Hint typechecking" $
             progress >> testTypeCheck cmdDataDir cmdTempDir hs
@@ -58,9 +66,13 @@ test CmdTest{..} main dataDir files = do
             progress >> testQuickCheck cmdDataDir cmdTempDir hs
 
         when (null files && not hasSrc) $ liftIO $ putStrLn "Warning, couldn't find source code, so non-hint tests skipped"
-        getIdeas
+        (,) <$> getIdeas <*> getBuiltins
     whenLoud $ mapM_ print ideas
-    return failures
+    when cmdGenerateSummary $ writeFile "builtin.md" (genBuiltinSummaryMd builtins)
+    case rpath of
+        Left refactorNotFound -> putStrLn $ unlines [refactorNotFound, "Refactoring tests skipped"]
+        _ -> pure ()
+    pure failures
 
 
 ---------------------------------------------------------------------
@@ -69,7 +81,7 @@ test CmdTest{..} main dataDir files = do
 -- Check all hints in the standard config files get sensible names
 testNames :: [Setting] -> Test ()
 testNames hints = sequence_
-    [ failed ["No name for the hint " ++ prettyPrint hintRuleLHS ++ " ==> " ++ prettyPrint hintRuleRHS]
+    [ failed ["No name for the hint " ++ unsafePrettyPrint hintRuleLHS ++ " ==> " ++ unsafePrettyPrint hintRuleRHS]
     | SettingMatchExp x@HintRule{..} <- hints, hintRuleName == defaultHintName]
 
 
