@@ -2,6 +2,19 @@
 -- | Translate the hints to Haskell and run with GHC.
 module Test.Translate(testTypeCheck, testQuickCheck) where
 
+import Config.Type
+import Control.Exception.Extra
+import Control.Monad.IO.Class
+import Test.Util
+
+testTypeCheck :: FilePath -> FilePath -> [[Setting]] -> Test ()
+testTypeCheck _ _ _ = liftIO $ errorIO "Test.Translate is disabled."
+
+-- | Given a set of hints, do all the HintRule hints satisfy QuickCheck
+testQuickCheck :: FilePath -> FilePath -> [[Setting]] -> Test ()
+testQuickCheck _ _ _ = liftIO $ errorIO "Test.Translate is disabled."
+
+{-
 import Control.Monad
 import Control.Monad.IO.Class
 import Data.List.Extra
@@ -10,6 +23,8 @@ import Data.Maybe
 import System.Process
 import System.Exit
 import System.FilePath
+import Language.Haskell.Exts.Util(FreeVars, freeVars)
+import qualified Data.Set as Set
 
 import Config.Type
 import HSE.All
@@ -19,10 +34,10 @@ import Test.Util
 runMains :: FilePath -> FilePath -> [String] -> Test ()
 runMains datadir tmpdir xs = do
     res <- liftIO $ (if tmpdir == "" then withTempDir else ($ tmpdir)) $ \dir -> do
-        ms <- forM (zip [1..] xs) $ \(i,x) -> do
+        ms <- forM (zipFrom 1 xs) $ \(i,x) -> do
             let m = "I" ++ show i
             writeFile (dir </> m <.> "hs") $ replace "module Main" ("module " ++ m) x
-            return m
+            pure m
         writeFile (dir </> "Main.hs") $ unlines $
             ["import qualified " ++ m | m <- ms] ++
             ["main = do"] ++
@@ -46,13 +61,15 @@ wrap f datadir tmpdir hints = runMains datadir tmpdir [unlines $ body [x | Setti
             ["{-# LANGUAGE NoMonomorphismRestriction, ExtendedDefaultRules, ScopedTypeVariables, DeriveDataTypeable #-}"
             ,"{-# LANGUAGE FlexibleInstances, UndecidableInstances, OverlappingInstances #-}"
             ,"module Main(main) where"] ++
-            concat [map (prettyPrint . hackImport) $ scopeImports $ hintRuleScope x | x <- take 1 xs] ++
+            -- concat [map (prettyPrint . hackImport) $ scopeImports $ hintRuleScope x | x <- take 1 xs] ++
             f xs
 
+        {-
         -- Hack around haskell98 not being compatible with base anymore
         hackImport i@ImportDecl{importAs=Just a,importModule=b}
             | prettyPrint b `elem` words "Maybe List Monad IO Char" = i{importAs=Just b,importModule=a}
         hackImport i = i
+        -}
 
 
 ---------------------------------------------------------------------
@@ -61,10 +78,10 @@ wrap f datadir tmpdir hints = runMains datadir tmpdir [unlines $ body [x | Setti
 toTypeCheck :: [HintRule] -> [String]
 toTypeCheck hints =
     ["import HLint_TypeCheck hiding(main)"
-    ,"main = return ()"] ++
+    ,"main = pure ()"] ++
     ["{-# LINE " ++ show (startLine $ ann rhs) ++ " " ++ show (fileName $ ann rhs) ++ " #-}\n" ++
      prettyPrint (PatBind an (toNamed $ "test" ++ show i) bod Nothing)
-    | (i, HintRule _ _ _ lhs rhs side _notes  _ghcScope  _ghcLhs _ghcRhs _ghcSide) <- zip [1..] hints, "noTypeCheck" `notElem` vars (maybeToList side)
+    | (i, HintRule _ _ lhs rhs side _notes  _ghcScope  _ghcLhs _ghcRhs _ghcSide) <- zipFrom 1 hints, "noTypeCheck" `notElem` vars (maybeToList side)
     , let vs = map toNamed $ nubOrd $ filter isUnifyVar $ vars lhs ++ vars rhs
     , let inner = InfixApp an (Paren an lhs) (toNamed "==>") (Paren an rhs)
     , let bod = UnGuardedRhs an $ if null vs then inner else Lambda an vs inner]
@@ -88,7 +105,7 @@ toQuickCheck hints =
                 Let an (BDecls an [PatBind an (toNamed "t") (UnGuardedRhs an bod) Nothing]) $
                 (toNamed "test" `app` str (fileName $ ann rhs) `app` int (startLine $ ann rhs) `app`
                  str (prettyPrint lhs ++ " ==> " ++ prettyPrint rhs)) `app` toNamed "t"
-            | (i, HintRule _ _ _ lhs rhs side note _ghcScope _ghcLhs _ghcRhs _ghcSide) <- zip [1..] hints, "noQuickCheck" `notElem` vars (maybeToList side)
+            | (i, HintRule _ _ lhs rhs side note _ghcScope _ghcLhs _ghcRhs _ghcSide) <- zipFrom 1 hints, "noQuickCheck" `notElem` vars (maybeToList side)
             , let vs = map (restrict side) $ nubOrd $ filter isUnifyVar $ vars lhs ++ vars rhs
             , let op = if any isRemovesError note then "?==>" else "==>"
             , let inner = InfixApp an (Paren an lhs) (toNamed op) (Paren an rhs)
@@ -104,3 +121,7 @@ toQuickCheck hints =
 isRemovesError :: Note -> Bool
 isRemovesError RemovesError{} = True
 isRemovesError _ = False
+
+vars :: FreeVars a => a -> [String]
+vars  = Set.toList . Set.map prettyPrint . freeVars
+-}
